<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Maple - PHP Web Application Framework
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @package     Maple
 * @author      TAKAHASHI Kunihiko <kunit@kunit.jp>
 * @copyright   2004-2006 The Maple Project
 * @license     http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version     CVS: $Id: ConverterManager.class.php,v 1.4 2006/02/11 19:18:22 kunit Exp $
 */

require_once 'converter/Converter.interface.php';

/**
 * Converter륯饹
 *
 * @package     Maple
 * @author      TAKAHASHI Kunihiko <kunit@kunit.jp>
 * @copyright   2004-2006 The Maple Project
 * @license     http://www.php.net/license/3_0.txt  PHP License 3.0
 * @access      public
 * @since       3.0.0
 */
class ConverterManager
{
    /**
     * @var Converterݻ
     *
     * @access  private
     * @since   3.0.0
     */
    var $_list;

    /**
     * 󥹥ȥ饯
     *
     * @access  public
     * @since   3.0.0
     */
    function ConverterManager()
    {
        $this->_list = array();
    }

    /**
     * ConvertԤ
     *
     * @param   array   $params Convert郎ä
     * @access  public
     * @since   3.0.0
     */
    function execute($params)
    {
        if (!is_array($params) || (count($params) < 1)) {
            return true;
        }

        // ConverterList
        ConverterManager::_buildConverterList($params);

        //
        // Convert¹
        //
        ConverterManager::_convert($params);

        return true;
    }

    /**
     * ConverterList
     *
     * @param   array   $params Convert郎ä
     * @access  private
     * @since   3.0.0
     */
    function _buildConverterList($params)
    {
        $log =& LogFactory::getLog();

        foreach ($params as $key => $value) {
            $key   = preg_replace("/\s+/", "", $key);
            $value = preg_replace("/\s+/", "", $value);

            if ($key == "") {
                $log->error("Converterλ꤬Ǥ", "ConverterManager#_buildConverterList");
                continue;
            }

            //
            // $key  attribute.name Υѥ
            //
            $keyArray = explode(".", $key);
            if (count($keyArray) != 2) {
                break;
            }
            $attribute = $keyArray[0];     // °̾
            $name      = $keyArray[1];     // Converter̾ 
            $name      = strtolower($name);

            $className = "Converter_" . ucfirst($name);
            $filename  = CONVERTER_DIR . "/${className}.class.php";

            if (!@file_exists($filename)) {
                $log->error("¸ߤƤʤConverterꤵƤޤ(${name})", "ConverterManager#_buildConverterList");
                continue;
            }

            //
            // Ʊ̾ConverterɲäƤ鲿⤷ʤ
            //
            if (isset($this->_list[$name]) && is_object($this->_list[$name])) {
                continue;
            }

            //
            // ֥Ȥ˼ԤƤ饨顼
            //
            include_once($filename);

            $converter =& new $className();

            if (!is_object($converter)) {
                $log->error("Converer˼Ԥޤ(${name})", "ConverterManager#_buildConverterList");
                return false;
            }

            $this->_list[$name] =& $converter;
        }
    }

    /**
     * Converter¹
     *
     * @param   array   $params Convert郎ä
     * @access  private
     * @since   3.0.0
     */
    function _convert($params)
    {
        $log =& LogFactory::getLog();

        foreach ($params as $key => $value) {
            $key   = preg_replace("/\s+/", "", $key);
            $value = preg_replace("/\s+/", "", $value);

            if ($key == "") {
                $log->error("Converterλ꤬Ǥ", "ConverterManager#_convert");
                continue;
            }

            //
            // $key  attribute.name Υѥ
            //
            $keyArray = explode(".", $key);
            if (count($keyArray) != 2) {
                break;
            }
            $attribute = $keyArray[0];     // °̾
            $name      = $keyArray[1];     // Converter̾ 
            $name      = strtolower($name);

            //
            // $value ˤConvertͤѿ̾åȤǤ
            //
            $newAttribute = $value;

            //
            // Converter
            //
            $converter =& $this->_list[$name];

            if (!is_object($converter)) {
                continue;
            }

            //
            // attribute * ꤵƤ
            // ꥯȥѥ᡼ƤѴоݤȤʤ
            //
            $container =& DIContainerFactory::getContainer();
            $request =& $container->getComponent("Request");

            if ($attribute == '*') {
                $attribute = join(",", array_keys($request->getParameters()));
            }

            if (preg_match("/,/", $attribute)) {
                $attributes = array();
                foreach (explode(",", $attribute) as $param) {
                    if ($param) {
                       $attributes[$param] = $request->getParameter($param);
                    }
                }
            } else {
                $attributes = $request->getParameter($attribute);
            }

            //
            // ConverterŬ
            //
            $result = $converter->convert($attributes);

            if ($newAttribute != "") {
                $request->setParameter($newAttribute, $result);
            } else {
                if (is_array($attributes)) {
                    foreach ($result as $key => $value) {
                        if ($key) {
                            $request->setParameter($key, $value);
                        }
                    }
                } else {
                    $request->setParameter($attribute, $result);
                }
            }
        }
    }
}
?>
