<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Maple - PHP Web Application Framework
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @package     Maple
 * @author      TAKAHASHI Kunihiko <kunit@kunit.jp>
 * @copyright   2004-2006 The Maple Project
 * @license     http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version     CVS: $Id: DIContainer.class.php,v 1.7 2006/02/11 19:18:22 kunit Exp $
 */

require_once 'core/BeanUtils.class.php';

/**
 * Dependency Injection¸륯饹
 *
 * бΤSetter InjectionΤ
 *
 * @package     Maple
 * @author      TAKAHASHI Kunihiko <kunit@kunit.jp>
 * @copyright   2004-2006 The Maple Project
 * @license     http://www.php.net/license/3_0.txt  PHP License 3.0
 * @access      public
 * @since       3.0.0
 */
class DIContainer
{
    /**
     * @var ContainerȤƴ륤󥹥󥹤Ǽ
     *
     * @access  private
     * @since   3.0.0
     */
    var $_components;

    /**
     * @var Ƥݻ
     *
     * @access  private
     * @since   3.0.0
     */
    var $_config;

    /**
     * @var ƥ饹°
     *
     * @access  private
     * @since   3.0.0
     */
    var $_attributes;

    /**
     * 󥹥ȥ饯
     *
     * @access  public
     * @since   3.0.0
     */
    function DIContainer()
    {
        $this->_components = array();
        $this->_config     = array();
        $this->_attributes = array();
    }

    /**
     * ե򸵤DIContainerȤΩƤ
     *
     * @param   string  $filename   ե̾
     * @return  boolean ContainerȤΩƤɤ
     * @access  public
     * @since   3.0.0
     */
    function create($filename)
    {
        $log =& LogFactory::getLog();

        //
        // եɹ
        //
        if (!$this->_parseConfig($filename)) {
            $log->error("եɤ߹ߤ˼Ԥޤ($filename)", "DIContainer#create");
            return false;
        }

        //
         // 򸵤Component
        //
        if (!$this->_buildComponents()) {
            $log->error("Component˼Ԥޤ", "DIContainer#create");
            return false;
        }

        //
        // Component°Injection
        //
        if (!$this->_injectAttributes()) {
            $log->error("°Inject˼Ԥޤ", "DIContainer#create");
            return false;
        }

        return true;
    }

    /**
     * եɤ߹
     *
     * @param   string  $filename   ե̾
     * @return  boolean ե뤬ɤ߹᤿(true/false)
     * @access  private
     * @since   3.0.0
     */
    function _parseConfig($filename)
    {
        if (!@file_exists($filename)) {
            return false;
        }

        $config = parse_ini_file($filename, TRUE);

        if (!isset($config) || (count($config) < 1)) {
            return false;
        }

        $this->_config = $config;

        return true;
    }

    /**
     * 򸵤Component
     *
     * @return  boolean ƤComponentǤ(true/false)
     * @access  private
     * @since   3.0.0
     */
    function _buildComponents()
    {
        $log =& LogFactory::getLog();

        //
        // ʬȤ'container'Ȥ̾ϿƤ
        //
        $this->register($this, 'container');

        foreach ($this->_config as $name => $body) {
            list ($alias, $class) = preg_split("/:/", $name);

            $this->_attributes[$alias] = array();

            foreach ($body as $key => $values) {
                $this->_attributes[$alias][$key] = preg_split("/,/", $values);
            }
    
            //
            // ꤵƤ륯饹̾ä饨顼
            //
            if (!preg_match("/^[0-9a-zA-Z.]+$/", $class)) {
                $log->error("饹̾Ǥ($class)", "DIContainer#_buildComponents");
                return false;
            }

            list ($className, $filename) = DIContainer::makeNames($class, true);

            if (!$className) {
                return false;
            }

            //
            // ֥Ȥ˼ԤƤ饨顼
            //
            include_once($filename);

            $component =& new $className();

            if (!is_object($component)) {
                $log->error("֥Ȥ˼Ԥޤ($className)", "DIContainer#_buildComponents");
                return false;
            }

            $this->register($component, $alias);
        }

        return true;
    }

    /**
     * 򸵤ComponentͤInjection
     *
     * @return  boolean Injection(true/false)
     * @access  private
     * @since   3.0.0
     */
    function _injectAttributes()
    {
        foreach ($this->_config as $name => $body) {
            list ($alias, $class) = explode(":", $name);

            $attributes = array();

            foreach ($this->_attributes[$alias] as $key => $values) {
                foreach ($values as $value) {
                    $value = trim($value);
                    if (preg_match("/^ref:/", $value)) {
                        $value = preg_replace("/^ref:/", "", $value);
                        if (!isset($this->_components[$value])) {
                            return false;
                        }
                        $attributes[$key][] =& $this->_components[$value];
                    } else {
                        $attributes[$key][] = $value;
                    }
                }

                if (isset($attributes[$key]) &&
                    (count($attributes[$key]) == 1)) {
                    $work =& $attributes[$key][0];
                    unset($attributes[$key]);
                    $attributes[$key] =& $work;
                }
            }

            $this->setAttributes($this->_components[$alias], $attributes);
        }

        return true;
    }

    /**
     * Component°Injection
     *
     * @param   Object  $component  ComponentΥ󥹥
     * @param   array   $attributes ͤä
     * @access  public
     * @since   3.0.0
     */
    function setAttributes(&$component, $attributes)
    {
        BeanUtils::setAttributes($component, $attributes);
    }

    /**
     * ContainerComponentΥ󥹥󥹤򥻥å
     *
     * @param   Object  $component  ComponentΥ󥹥
     * @param   string  $name   Component̾
     * @access  public
     * @since   3.0.0
     */
    function register(&$component, $name = '')
    {
        if (!is_object($component)) {
            return;
        }

        if ($name == '') {
            $name = get_class($component);
        }

        $this->_components[$name] =& $component;
    }

    /**
     * ե򸵤ComponentΥ󥹥󥹤ֵ
     *
     * @param   string  $name   Component̾
     * @return  Object  ComponentΥ󥹥
     * @access  public
     * @since   3.0.0
     */
    function &getComponent($name)
    {
        $component = NULL;

        if (isset($this->_components[$name])) {
            $component =& $this->_components[$name];
        }

        return $component;
    }

    /**
     * ComponentΥ饹̾ӥեѥֵѤ
     *
     * @param   string  $class  Component̾
     * @param   boolean $check  ե¸ߥå򤹤뤫ɤ
     * @return  array   ComponentΥ饹̾ȥեѥ
     * @access  public
     * @since   3.0.0
     */
    function makeNames($class, $check = false)
    {
        $pathList   = explode(".", $class);
        $ucPathList = array_map('ucfirst', $pathList);

        $basename = ucfirst(array_pop($pathList));

        $classPath = join("/", $pathList);
        $className = join("_", $ucPathList);
        $filename = COMPONENT_DIR . "/${classPath}/${basename}.class.php";

        if (!$check) {
            return array($className, $filename);
        }

        if (!@file_exists($filename)) {
            $filename = COMPONENT_DIR . "/${classPath}/${className}.class.php";
            if (!@file_exists($filename)) {
                $log =& LogFactory::getLog();
                $log->error("饹ե뤬ޤ($filename)", "DIContainer#makeNames");
                $className = null;
                $filename  = null;
            }
        }

        return array($className, $filename);
    }
}
?>
