<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Maple - PHP Web Application Framework
 *
 * PHP versions 4 and 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @package     Maple
 * @author      TAKAHASHI Kunihiko <kunit@kunit.jp>
 * @copyright   2004-2006 The Maple Project
 * @license     http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version     CVS: $Id: FilterChain.class.php,v 1.5 2006/02/11 19:18:22 kunit Exp $
 */

require_once 'filter/Filter.interface.php';

/**
 * Filterݻ륯饹
 *
 * @package     Maple
 * @author      TAKAHASHI Kunihiko <kunit@kunit.jp>
 * @copyright   2004-2006 The Maple Project
 * @license     http://www.php.net/license/3_0.txt  PHP License 3.0
 * @access      public
 * @since       3.0.0
 */
class FilterChain
{
    /**
     * @var Filterݻ
     *
     * @access  private
     * @since   3.0.0
     */
    var $_list;

    /**
     * @var Filterΰ֤ݻ
     *
     * @access  private
     * @since   3.0.0
     */
    var $_position;

    /**
     * @var ߼¹ԤƤFilterΰ֤ݻ
     *
     * @access  private
     * @since   3.0.0
     */
    var $_index;

    /**
     * 󥹥ȥ饯
     *
     * @access  public
     * @since   3.0.0
     */
    function FilterChain()
    {
        $this->_list     = array();
        $this->_position = array();
        $this->_index    = -1;
    }

    /**
     * FilterChainκǸFilterɲ
     *
     * @param   string  $name   FilterΥ饹̾
     * @param   string  $alias  FilterΥꥢ̾
     * @access  public
     * @since   3.0.0
     */
    function add($name, $alias = '')
    {
        $log =& LogFactory::getLog();

        //
        // ꥢ̾ꤵƤʤϥ饹̾򥻥å
        //
        if (empty($alias)) {
            $alias = $name;
        }

        //
        // Filterμ¹Ԥ˻ϤޤäƤ饨顼(¹Ըɲäϥ顼)
        //
        if ($this->_index > -1) {
            $log->error("¹ԸFilterɲäƤޤ(${name}[alias:${alias}])", "FilterChain#add");
            return false;
        }

        //
        // FilterΥ饹̾ä饨顼
        //
        if (!preg_match("/^[0-9a-zA-Z_]+$/", $name)) {
            $log->error("FilterꤵƤޤ(${name}[alias:${alias}])", "FilterChain#add");
            return false;
        }

        //
        // ե뤬¸ߤƤʤХ顼
        //
        $className = "Filter_" . ucfirst($name);
        $filename  = FILTER_DIR . "/${className}.class.php";

        if (!@file_exists($filename)) {
            $log->error("¸ߤƤʤFilterꤵƤޤ(${name}[alias:${alias}])", "FilterChain#add");
            return false;
        }

        //
        // Ʊ̾FilterɲäƤ鲿⤷ʤ
        //
        if (isset($this->_list[$alias]) && is_object($this->_list[$alias])) {
            $log->info("FilterϴϿƤޤ(${name}[alias:${alias}])", "FilterChain#add");
            return true;
        }

        //
        // ֥Ȥ˼ԤƤ饨顼
        //
        include_once($filename);

        $filter =& new $className();

        if (!is_object($filter)) {
            $log->error("Filter˼Ԥޤ(${name}[alias:${alias}])", "FilterChain#add");
            return false;
        }

        $this->_list[$alias] =& $filter;
        $this->_position[]   =  $alias;

        return true;
    }

    /**
     * FilterChain򥯥ꥢ
     *
     * @access  public
     * @since   3.0.0
     */
    function clear()
    {
        $this->_list     = array();
        $this->_position = array();
        $this->_index    = -1;
    }

    /**
     * FilterChainĹֵ
     *
     * @return  integer FilterChainĹ
     * @access  public
     * @since   3.0.0
     */
    function getSize()
    {
        return count($this->_list);
    }

    /**
     * ߤFilterֵ̾
     *
     * @return  string  Filter̾
     * @access  public
     * @since   3.0.0
     */
    function getCurFilterName()
    {
        if (isset($this->_position[$this->_index])) {
            return $this->_position[$this->_index];
        }
    }

    /**
     * ꤵ줿̾Filterֵ
     *
     * @return  Object  FilterΥ󥹥
     * @access  public
     * @since   3.0.0
     */
    function &getFilterByName($name)
    {
        $log =& LogFactory::getLog();

        if ($name == "") {
            $log->warn("Ǥ", "FilterChain#getFilterByName");
            return false;
        }

        if (!isset($this->_list[$name]) || !is_object($this->_list[$name])) {
            $log->error("ꤵ줿FilterϿƤޤ(${name})", "FilterChain#getFilterByName");
            return false;
        }

        $filter =& $this->_list[$name];

        if (!is_object($filter)) {
            $log->error("Filterμ˼Ԥޤ(${name})", "FilterChain#getFilterByname");
            return false;
        }

        return $filter;
    }

    /**
     * ꤷFilter°򥻥å
     *
     * @param   string  $name   Filter̾
     * @param   array   $attributes åȤ°()
     * @access  public
     * @since   3.0.0
     */
    function setAttributes($name, $attributes)
    {
        $filter =& $this->getFilterByname($name);

        if (!is_object($filter)) {
            $log->error("Filterμ˼Ԥޤ(${name})", "FilterChain#setAttributes");
            return false;
        }

        return $filter->setAttributes($attributes);
    }

    /**
     * FilterChainȤΩƤ
     *
     * @param   Object  $config ConfigUtilsΥ󥹥
     * @access  public
     * @since   3.0.0
     */
    function build(&$config)
    {
        $log =& LogFactory::getLog();

        foreach ($config->getConfig() as $section => $value) {
            $sections = explode(':', $section);
            $filterName = $sections[0]; // ե륿̾
            if (isset($sections[1]) && $sections[1]) { // ȯưREQUEST_METHOD
                $method = strtoupper($sections[1]);
            } else {
                $method = 'BOTH';
            }
            if (isset($sections[2]) && $sections[2]) { // ꥢ̾
                $alias = $sections[2];
            } else {
                $alias = $filterName;
            }

            if (($method == 'BOTH') ||
                ($method == $_SERVER['REQUEST_METHOD'])) {
                $filterConfig =& $config->getSectionConfig($section);
                if (!$this->add($filterName, $alias)) {
                    $log->error("FilterChainؤɲä˼Ԥޤ(${section})", "FilterChain#build");
                    return false;
                }
                if (is_array($filterConfig) && (count($filterConfig) > 0)) {
                    $this->setAttributes($alias, $filterConfig);
                }
            }
        }

        return true;
    }

    /**
     * FilterChainμFilter¹
     *
     * Υ᥽åɤϥ饹᥽å
     *
     * @access  public
     * @since   3.0.0
     */
    function execute()
    {
        $log =& LogFactory::getLog();

        if ($this->getSize() < 1) {
            $log->error("FilterɲäƤޤ", "FilterChain#execute");
            return false;
        }

        if ($this->_index < ($this->getSize() - 1)) {
            $this->_index++;

            $name = $this->getCurFilterName();
            $filter =& $this->getFilterByname($name);

            if (!is_object($filter)) {
                $log->error("Filterμ˼Ԥޤ(${name})", "FilterChain#execute");
                return false;
            }

            return $filter->execute();
        }

        return true;
    }
}
?>
